package com.jeeplus.finance.proofread.controller;

import com.jeeplus.core.service.TreeService;
import com.jeeplus.finance.proofread.domain.ProofreadType;
import com.jeeplus.finance.proofread.mapper.ProofreadTypeMapper;
import com.jeeplus.finance.proofread.service.ProofreadTypeForTreeDataService;
import com.jeeplus.finance.proofread.service.ProofreadTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

@RestController
@Api(tags ="项目登记校对")
@RequestMapping(value = "/proofread/type")
public class ProofreadTypeController {

    @Resource
    private ProofreadTypeService service;

    @Resource
    private ProofreadTypeMapper mapper;

    @Resource
    private ProofreadTypeForTreeDataService treeDataService;

    /**
     * 列表查询
     * @param type
     * @return
     */
    @ApiOperation(value = "列表查询")
    @GetMapping("/list")
    public ResponseEntity<List<ProofreadType>> list(ProofreadType type) {
        List<ProofreadType> list = service.list(type);
        return ResponseEntity.ok(list);
    }

    /**
     * 新增/修改
     * @param info
     * @return
     */
    @ApiOperation(value = "新增/修改")
    @PostMapping("/save")
    public ResponseEntity<String> save(@RequestBody ProofreadType info) {
        String s = service.saveType(info);
        return ResponseEntity.ok(s);
    }

    /**
     * 根据id查询
     * @param id
     * @return
     */
    @ApiOperation(value = "根据id查询")
    @GetMapping("/findById")
    public ResponseEntity<ProofreadType> findById(String id) {
        ProofreadType info = mapper.selectById(id);
        return ResponseEntity.ok(info);
    }

    /**
     * 根据id删除
     * @param id
     * @return
     */
    @ApiOperation(value = "根据id删除")
    @GetMapping("/deleteById")
    public ResponseEntity<String> deleteById(String id) {
        mapper.deleteById(id);
        return ResponseEntity.ok("操作完成");
    }

    /**
     * 查询树形
     * @param extId 排除的ID
     * @return
     */
    @ApiOperation(value = "查询树形")
    @GetMapping("/treeData")
    public ResponseEntity<List<ProofreadType>> treeData(@RequestParam(required = false) String extId, @RequestParam(required = false) String type) throws Exception{
        List<ProofreadType> infos = treeDataService.treeDataForType(extId, type);
        return ResponseEntity.ok(infos);
    }

}
