package com.jeeplus.finance.reimbursement.reimbursementInfo.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.jeeplus.common.excel.ExcelOptions;
import com.jeeplus.common.excel.ExportMode;
import com.jeeplus.common.utils.ResponseUtil;
import com.jeeplus.finance.projectRecords.Utils.EasyPoiUtil;
import com.jeeplus.finance.reimbursement.reimbursementInfo.domain.ReimbursementInfo;
import com.jeeplus.finance.reimbursement.reimbursementInfo.service.ReimbursementInfoService;
import com.jeeplus.finance.reimbursement.reimbursementInfo.service.dto.*;
import com.jeeplus.finance.reimbursementApproval.approvalInfo.domain.ProgramProjectListInfo;
import com.jeeplus.logging.annotation.ApiLog;
import com.jeeplus.utils.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
@Api(tags ="报销申请")
@RequestMapping(value = "/reimbursement/info")
public class ReimbursementInfoController {

    @Resource
    private ReimbursementInfoService service;

    /**
     * 列表查询
     * @param dto
     * @return
     */
    @ApiOperation(value = "列表查询")
    @GetMapping("/list")
    public ResponseEntity<IPage<RetureListDto>> list(Page<RetureListDto> page, QueryListDto dto) throws Exception{
        IPage<RetureListDto> iPage = service.list(page, dto);
        return ResponseEntity.ok(iPage);
    }

    /**
     * 新增/修改
     * @param dto
     * @return
     */
    @ApiOperation(value = "新增、修改")
    @PostMapping("/save")
    public ResponseEntity<String> save(@RequestBody SaveInfoDto dto) throws Exception{
        ReimbursementInfo s = service.save(dto);
        return ResponseUtil.newInstance().add("businessTable", "reimbursement_info")
                .add("businessId", s.getId())
                .add("no", s.getNo())
                .ok ("操作成功");
    }

    /**
     * 删除
     * @param id
     * @return
     */
    @ApiOperation(value = "删除")
    @GetMapping("/remove")
    public ResponseEntity<String> remove(@RequestParam String id){
        String s = service.remove(id);
        return ResponseEntity.ok(s);
    }

    /**
     * 查询
     * @param id
     * @return
     */
    @ApiOperation(value = "查询")
    @GetMapping("/findById")
    public ResponseEntity<SaveInfoDto> findById(@RequestParam("id") String id) throws Exception{
        SaveInfoDto dto = service.findById(id);
        return ResponseEntity.ok(dto);
    }

    @ApiOperation(value = "发票号判断")
    @GetMapping("/checkNumber")
    public ResponseEntity<Boolean> checkNumber(@RequestParam("number") String number) {
        Boolean is = service.checkNumber(number);
        return ResponseEntity.ok(is);
    }

    /**
     * 根据id修改状态status
     */
    @ApiOperation(value = "根据id修改状态status")
    @PostMapping(value = "updateStatusById")
    public void updateStatusById(@RequestBody RetureListDto dto) {
        service.updateStatusById(dto);
    }

    @ApiOperation(value = "用户树形")
    @GetMapping(value = "/userTree")
    public ResponseEntity<List<TreeUserDto>> userTree(@RequestParam String name) {
        List<TreeUserDto> list = service.userTree(name);
        return ResponseEntity.ok(list);
    }

    @ApiOperation(value = "用户树形")
    @GetMapping(value = "/userTreeNew")
    public ResponseEntity<List<TreeUserDto>> userTreeNew(@RequestParam String name) {
        List<TreeUserDto> list = service.userTreeNew(name);
        return ResponseEntity.ok(list);
    }


    @ApiLog(value = "评估报销数据")
    @GetMapping("exportFile")
    @ApiOperation(value = "评估报销数据")
    public void exportFile(QueryListDto cwDTO, Page <RetureListDto> page, ExcelOptions options, HttpServletResponse response) throws Exception {
        String fileName = options.getFilename();
        String sheetName = options.getSheetName();
        List<RetureListDto> result = new ArrayList<>();
        if ( ExportMode.current.equals ( options.getMode() ) ) {
            result = service.list (page,cwDTO).getRecords();
        } else if (ExportMode.selected.equals ( options.getMode() )) {
            result = service.list (page,cwDTO).getRecords().stream ( ).filter ( item ->
                    options.getSelectIds ( ).contains ( item.getId ( ) )
            ).collect ( Collectors.toList ( ) );
        } else {
            page.setSize (-1);
            page.setCurrent (0);
            result = service.list (page,cwDTO).getRecords();
        }
        EasyPoiUtil.exportExcel ( result, sheetName,  sheetName, RetureListDto.class, fileName, response );

    }


    @ApiLog(value = "评估电子发票报销数据")
    @GetMapping("exportInvoiceReimbursementFile")
    @ApiOperation(value = "评估电子发票报销数据")
    public void exportInvoiceReimbursementFile(QueryListDto cwDTO, Page <RetureListDto> page, ExcelOptions options, HttpServletResponse response) throws Exception {
        String fileName = "评估电子发票报销数据表";
        String sheetName = "评估电子发票报销列表";
        List<RetureListDto> result = new ArrayList<>();
        if ( ExportMode.current.equals ( options.getMode() ) ) {
            result = service.list (page,cwDTO).getRecords();
        } else if (ExportMode.selected.equals ( options.getMode() )) {
            result = service.list (page,cwDTO).getRecords().stream ( ).filter ( item ->
                    options.getSelectIds ( ).contains ( item.getId ( ) )
            ).collect ( Collectors.toList ( ) );
        } else {
            page.setSize (-1);
            page.setCurrent (0);
            if(StringUtils.isBlank(cwDTO.getReimbursementType())){
                cwDTO.setReimbursementType("1");
            }
            result = service.list (page,cwDTO).getRecords();
        }
        List<String> idList = Lists.newArrayList();
        for (RetureListDto listDto : result) {
            idList.add(listDto.getId());
        }
        List<ReimbursementAmountInfoDTO> invoiceReimbursementAmountList = Lists.newArrayList();
        if(idList.size()>0){
            invoiceReimbursementAmountList = service.getInvoiceReimbursementAmountList(idList);
        }
        for (int i = 1;i <=invoiceReimbursementAmountList.size();i ++){
            ReimbursementAmountInfoDTO reimbursementAmountInfoDTO = invoiceReimbursementAmountList.get(i - 1);
            reimbursementAmountInfoDTO.setSerialNumber(i);
        }

        EasyPoiUtil.exportExcel ( invoiceReimbursementAmountList, sheetName,  sheetName, ReimbursementAmountInfoDTO.class, fileName, response );

    }

    @ApiOperation(value = "关联报销")
    @GetMapping(value = "/relationReimbursementList")
    public ResponseEntity<IPage<RetureListDto>> relationReimbursementList(Page<RetureListDto> page, @RequestParam("id") String id) {
        IPage<RetureListDto> listDtoIPage = service.relationReimbursementList(page, id);
        return ResponseEntity.ok(listDtoIPage);
    }

    @ApiOperation(value = "报销审核")
    @PostMapping(value = "/auditFunc")
    public Map<String,Object> auditFunc(@RequestBody SaveInfoDto saveInfoDto) {
        Map<String, Object> res = service.auditFunc(saveInfoDto);
        return res;
    }

    @ApiOperation(value = "列表查询")
    @GetMapping("/queryByProIds")
    public ResponseEntity<List<ProgramProjectListInfo>> queryByProIds(String ids){
        List<ProgramProjectListInfo> cwProjectRecords = service.queryByProIds(ids);
        return ResponseEntity.ok(cwProjectRecords);
    }

    /**
     * 根据发票号查询是否已经或正在报销
     * @return
     */
    @ApiLog(value = "根据发票号查询是否已经或正在报销")
    @GetMapping("getEffectiveDataByInvoiceNumber")
    @ApiOperation(value = "根据发票号查询是否已经或正在报销")
    public Map<String,Object> getEffectiveDataByInvoiceNumber(String invoiceNumber,String id){
        Map map = new HashMap();
        Integer count = service.getEffectiveDataByInvoiceNumber(invoiceNumber,id);
        if(count>0){
            map.put("success",false);
            map.put("message","发票号：" + invoiceNumber + " 已发起或完成报销，请勿重复报销");
        }else{
            map.put("success",true);
        }
        return map;
    }
}
